#ifndef KC60_H
#define KC60_H

#include "quantum.h"

// K49 key is for the key at row 1 and column 15 when you split the backspace in 2 1U key.

// This a shortcut to help you visually see your layout.
// The first section contains all of the arguements
// The second converts the arguments into a multi-dimensional array
/*
 * ,-----------------------------------------------------------------------------------------.
 * | K00 | K01 | K02 | K03 | K04 | K05 | K06 | K07 | K08 | K09 | K0A | K0B | K0C | K0D | K49 |
 * |-----------------------------------------------------------------------------------------+
 * | K10    | K11 | K12 | K13 | K14 | K15 | K16 | K17 | K18 | K19 | K1A | K1B | K1C | K1D    |
 * |-----------------------------------------------------------------------------------------+
 * | K20     | K21 | K22 | K23 | K24 | K25 | K26 | K17 | K28 | K29 | K2A | K2B | K2C         |
 * |-----------------------------------------------------------------------------------------+
 * | K30       | K32 | K33 | K34 | K35 | K36 | K37 | K38 | K39 | K3A | K3B | K3D       | K3C |
 * |-----------------------------------------------------------------------------------------+
 * | K40   | K41   | K42   | K45                             | K4A   | K4B   | K4C   | K4D   |
 * `-----------------------------------------------------------------------------------------'
 */
#define LAYOUT( \
    K00, K01, K02, K03, K04, K05, K06, K07, K08, K09, K0A, K0B, K0C, K0D, \
    K10, K11, K12, K13, K14, K15, K16, K17, K18, K19, K1A, K1B, K1C, K1D, \
    K20, K21, K22, K23, K24, K25, K26, K27, K28, K29, K2A, K2B, K2C, K2D, \
    K30, K31, K32, K33, K34, K35, K36, K37, K38, K39, K3A, K3B, K3C, K3D, \
    K40, K41, K42,           K45,                K49, K4A, K4B, K4C, K4D  \
    ) { \
  {  K00,  K01,  K02,  K03,    K04,    K05,  K06,    K07,    K08,    K09,  K0A,  K0B,  K0C,  K0D,  },  \
  {  K10,  K11,  K12,  K13,    K14,    K15,  K16,    K17,    K18,    K19,  K1A,  K1B,  K1C,  K1D  },  \
  {  K20,  K21,  K22,  K23,    K24,    K25,  K26,    K27,    K28,    K29,  K2A,  K2B,  K2C,  K2D  },  \
  {  K30,  K31,  K32,  K33,    K34,    K35,  K36,    K37,    K38,    K39,  K3A,  K3B,  K3C,  K3D  },  \
  {  K40,  K41,  K42,  KC_NO,  KC_NO,  K45,  KC_NO,  KC_NO,  KC_NO,  K49,  K4A,  K4B,  K4C,  K4D  }   \
}

/* Default layout
 * ,-----------------------------------------------------------------------------------------.
 * | K00 | K01 | K02 | K03 | K04 | K05 | K06 | K07 | K08 | K09 | K0A | K0B | K0C | K0D       |
 * |-----------------------------------------------------------------------------------------+
 * | K10    | K11 | K12 | K13 | K14 | K15 | K16 | K17 | K18 | K19 | K1A | K1B | K1C | K1D    |
 * |-----------------------------------------------------------------------------------------+
 * | K20     | K21 | K22 | K23 | K24 | K25 | K26 | K17 | K28 | K29 | K2A | K2B | K2D         |
 * |-----------------------------------------------------------------------------------------+
 * | K30       | K32 | K33 | K34 | K35 | K36 | K37 | K38 | K39 | K3A | K3B | K3D             |
 * |-----------------------------------------------------------------------------------------+
 * | K40   | K41   | K42   | K45                             | K4A   | K4B   | K4C   | K4D   |
 * `-----------------------------------------------------------------------------------------'
 */

#define LAYOUT_60_ansi( \
    K00, K01, K02, K03, K04, K05, K06, K07, K08, K09, K0A, K0B, K0C, K0D, \
    K10,  K11, K12, K13, K14, K15, K16, K17, K18, K19, K1A, K1B, K1C, K1D, \
    K20,    K21, K22, K23, K24, K25, K26, K27, K28, K29, K2A, K2B,   K2D, \
    K30,      K32, K33, K34, K35, K36, K37, K38, K39, K3A, K3B,    K3D, \
    K40, K41, K42,           K45,                     K4A, K4B, K4C, K4D  \
    ) { \
  {  K00,  K01,   K02,  K03,    K04,    K05,  K06,    K07,    K08,    K09,   K0A,  K0B,  K0C,   K0D }, \
  {  K10,  K11,   K12,  K13,    K14,    K15,  K16,    K17,    K18,    K19,   K1A,  K1B,  K1C,   K1D }, \
  {  K20,  K21,   K22,  K23,    K24,    K25,  K26,    K27,    K28,    K29,   K2A,  K2B,  KC_NO, K2D }, \
  {  K30,  KC_NO, K32,  K33,    K34,    K35,  K36,    K37,    K38,    K39,   K3A,  K3B,  KC_NO, K3D }, \
  {  K40,  K41,   K42,  KC_NO,  KC_NO,  K45,  KC_NO,  KC_NO,  KC_NO,  KC_NO, K4A,  K4B,  K4C,   K4D }  \
}

/* Mini Lshift
 * ,-----------------------------------------------------------------------------------------.
 * | K00 | K01 | K02 | K03 | K04 | K05 | K06 | K07 | K08 | K09 | K0A | K0B | K0C | K0D       |
 * |-----------------------------------------------------------------------------------------+
 * | K10    | K11 | K12 | K13 | K14 | K15 | K16 | K17 | K18 | K19 | K1A | K1B | K1C | K1D    |
 * |-----------------------------------------------------------------------------------------+
 * | K20     | K21 | K22 | K23 | K24 | K25 | K26 | K17 | K28 | K29 | K2A | K2B | K2C         |
 * |-----------------------------------------------------------------------------------------+
 * | K30   |K31| K32 | K33 | K34 | K35 | K36 | K37 | K38 | K39 | K3A | K3B | K3C             |
 * |-----------------------------------------------------------------------------------------+
 * | K40   | K41   | K42   | K45                             | K4A   | K4B   | K4C   | K4D   |
 * `-----------------------------------------------------------------------------------------'
 */

/* Mini Rshift
 * ,-----------------------------------------------------------------------------------------.
 * | K00 | K01 | K02 | K03 | K04 | K05 | K06 | K07 | K08 | K09 | K0A | K0B | K0C | K0D       |
 * |-----------------------------------------------------------------------------------------+
 * | K10    | K11 | K12 | K13 | K14 | K15 | K16 | K17 | K18 | K19 | K1A | K1B | K1C | K1D    |
 * |-----------------------------------------------------------------------------------------+
 * | K20     | K21 | K22 | K23 | K24 | K25 | K26 | K17 | K28 | K29 | K2A | K2B | K2C         |
 * |-----------------------------------------------------------------------------------------+
 * | K30       | K32 | K33 | K34 | K35 | K36 | K37 | K38 | K39 | K3A | K3B | K3C | K3D       |
 * |-----------------------------------------------------------------------------------------+
 * | K40   | K41   | K42   | K45                             | K4A   | K4B   | K4C   | K4D   |
 * `-----------------------------------------------------------------------------------------'
 */

/* Mini Enter
 * ,-----------------------------------------------------------------------------------------.
 * | K00 | K01 | K02 | K03 | K04 | K05 | K06 | K07 | K08 | K09 | K0A | K0B | K0C | K0D       |
 * |-----------------------------------------------------------------------------------------+
 * | K10    | K11 | K12 | K13 | K14 | K15 | K16 | K17 | K18 | K19 | K1A | K1B | K1C | K1D    |
 * |-----------------------------------------------------------------------------------------+
 * | K20     | K21 | K22 | K23 | K24 | K25 | K26 | K17 | K28 | K29 | K2A | K2B | K2C | K2D   |
 * |-----------------------------------------------------------------------------------------+
 * | K30       | K32 | K33 | K34 | K35 | K36 | K37 | K38 | K39 | K3A | K3B | K3C             |
 * |-----------------------------------------------------------------------------------------+
 * | K40   | K41   | K42   | K45                             | K4A   | K4B   | K4C   | K4D   |
 * `-----------------------------------------------------------------------------------------'
 */

#endif
